//+------------------------------------------------------------------+
//|                                                       MyHope.mq4 |
//|                      Copyright  2011, MetaQuotes Software Corp. |
//|                                        http://www.metaquotes.net |
//+------------------------------------------------------------------+
#property copyright "Copyright  2011, MetaQuotes Software Corp."
#property link      "http://www.metaquotes.net"

//--- input parameters
int       Magic;
extern double    LotSize=0;
extern bool      OpenNewTradeWithEachNewBar=false;
extern string    TradeComment = "";
extern bool      SaveScreenShotWithEachTrade=true;
extern int       ScreenShotWidth=1280;
extern int       ScreenShotHeight=900;
extern bool      VerifyWith1=true;
extern bool      VerifyWith2=true;
extern string TradingTime = "----------------------------";
extern bool      UseTimeLimits=false;
extern int       StartDay=1;
extern int       EndDay = 5;
extern int       StartHour = 9;
extern int       EndHour = 15;
extern int       StartMin=30;
extern int       EndMin=0;
int lastTime;
double ls;
string fibComment;
//+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+
int init()
  {
//----
   lastTime = iTime(NULL, 0, 0);
   TradeComment =  TradeComment + Period();
   Magic = Period()*13;
   if(LotSize == 0)
   {
      ls = AccountBalance() * 0.00001;
      if(ls < MarketInfo(Symbol(), MODE_MINLOT)) ls = MarketInfo(Symbol(), MODE_MINLOT);
      else if(ls > MarketInfo(Symbol(), MODE_MAXLOT)) ls =MarketInfo(Symbol(), MODE_MAXLOT);
   }
   else
   {
      ls = LotSize;
   }
   
//----
   return(0);
  }
  
void TakeScreenShot(int ticketNum)
{
   if(SaveScreenShotWithEachTrade)
   {
      WindowScreenShot(StringConcatenate(TradeComment, ticketNum, "-",Symbol(),Period(), ".jpg"), ScreenShotWidth, ScreenShotHeight);
   }
}
bool MatchUpFib(int time)
{
   fibComment = "";
   string objName;
   for(int i=0; i<ObjectsTotal(); i++)
   {
       objName = ObjectName(i);
      
       if(ObjectType(objName) == OBJ_FIBOTIMES)
       {
         int first =  ObjectGet(objName, OBJPROP_TIME1);
         int last =   ObjectGet(objName, OBJPROP_TIME2);
         int second = last - ((last - first)*0.382);
         int third = last - ((last - first)*0.618);

         if(time == first || time == second || time == third || time == last)
         {

               if(first == time) fibComment = StringConcatenate(fibComment, "FIB ALIGNMENT  = ", TimeToStr(first), "\r\n");
               else if(second == time)fibComment = StringConcatenate(fibComment, "FIB ALIGNMENT  = ", TimeToStr(second), "\r\n");
               else if(third == time)fibComment = StringConcatenate(fibComment, "FIB ALIGNMENT  = ", TimeToStr(third), "\r\n");
               else if(last == time)fibComment = StringConcatenate(fibComment, "FIB ALIGNMENT  = ", TimeToStr(last), "\r\n");

               fibComment = StringConcatenate(fibComment, "0 = ",TimeToStr(first), ", 0.382 = ", TimeToStr(second), ", 0.618 = ", TimeToStr(third), ", 1.0 = ", TimeToStr(last), "\r\n"); 
            
            return(true);
         }    
       }   
   }   
   return(false);
}
//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+------------------------------------------------------------------+
int deinit()
  {
//----
   
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| expert start function                                            |
//+------------------------------------------------------------------+
int start()
  {
   fibComment = "";
   int posSemaphore1Dn, posSemaphore1Up, posSemaphore2Dn, posSemaphore2Up, posSemaphore3Dn, posSemaphore3Up;
   double semaphore1Dn, semaphore1Up, semaphore2Dn, semaphore2Up, semaphore3Dn, semaphore3Up;

   double macD = iMACD(Symbol(), NULL, 12, 26, 9, PRICE_CLOSE, MODE_EMA, 0);
   posSemaphore1Dn = GlobalVariableGet(Symbol() + Period() + "TimeDn" + 1);
   posSemaphore2Dn = GlobalVariableGet(Symbol() + Period() + "TimeDn" + 2);
   posSemaphore3Dn = GlobalVariableGet(Symbol() + Period() + "TimeDn" + 3);
   posSemaphore1Up = GlobalVariableGet(Symbol() + Period() + "TimeUp" + 1);
   posSemaphore2Up = GlobalVariableGet(Symbol() + Period() + "TimeUp" + 2);
   posSemaphore3Up = GlobalVariableGet(Symbol() + Period() + "TimeUp" + 3);
   
   semaphore2Dn = GlobalVariableGet(Symbol() + Period() + "ItemDn" + 1);
   semaphore2Dn = GlobalVariableGet(Symbol() + Period() + "ItemDn" + 2);
   semaphore3Dn = GlobalVariableGet(Symbol() + Period() + "ItemDn" + 3);
   semaphore1Up = GlobalVariableGet(Symbol() + Period() + "ItemUp" + 1);
   semaphore2Up = GlobalVariableGet(Symbol() + Period() + "ItemUp" + 2);
   semaphore3Up = GlobalVariableGet(Symbol() + Period() + "ItemUp" + 3);
   
   bool buy = false;
   bool sell = false;
   double profit=0;
   bool allProfit = true;
   bool mtf = true;
   
  
   bool validSellSignal;
   bool validBuySignal;
   double totalLotSize = 0.0;
   double totalProfit = 0.0;
   
   for(int i=0; i<OrdersTotal(); i++)
   {
      if(OrderSelect(i, SELECT_BY_POS, MODE_TRADES) == true)
      {
         if(OrderSymbol() == Symbol() && OrderMagicNumber() == Magic)
         {
            totalLotSize += OrderLots();
            totalProfit += OrderProfit();
            if(OrderType() == OP_BUY)
            { 
               buy = true;
               if(posSemaphore3Dn > posSemaphore3Up)
               {
                  CloseAllOrders();
               }
            }
            if(OrderType() == OP_SELL)
            { 
               sell = true;
               if(posSemaphore3Dn < posSemaphore3Up)
               {
                  CloseAllOrders();
               }
            }
            profit += OrderProfit();
            if(OrderProfit() < 0.0)
            {
               allProfit = false;
            }
         }
      }

   }
   
   
   if(posSemaphore3Dn < posSemaphore3Up)// && MatchUpFib(posSemaphore3Up))//Buy trend
   {
      if((posSemaphore1Up > posSemaphore3Up && VerifyWith1) || (posSemaphore2Up > posSemaphore3Up  && VerifyWith2))//Trend Verified with first retraction
      {
         if(macD > 0)
         {
            validBuySignal = true;
         }
      } 
   }   
   
   if(posSemaphore3Dn > posSemaphore3Up)// && MatchUpFib(posSemaphore3Dn))//Sell Trend
   {
      if((posSemaphore1Dn > posSemaphore3Dn && VerifyWith1)|| (posSemaphore2Dn > posSemaphore3Dn && VerifyWith1))//Trend Verified with first retraction
      {
         if(macD < 0)
         {
            validSellSignal = true;
         }
      }
   }
   
      int ticketNum;
      if(!sell && validSellSignal)
      {
        if(IsInTime())
        {
           ticketNum = OrderSend(Symbol(), OP_SELL, ls, Bid, 3, 0, 0,TradeComment + " - Sell", Magic, 0, Red); 
           if(ticketNum > 0)
           {
              TakeScreenShot(ticketNum);
           } 
        }
      }
      else if(lastTime != iTime(NULL, 0, 0) && validSellSignal && allProfit && OpenNewTradeWithEachNewBar && IsInTime())
      {
         lastTime = iTime(NULL, 0, 0);
         ticketNum = OrderSend(Symbol(), OP_SELL, ls, Bid, 3, 0, 0,TradeComment + " - Sell", Magic, 0, Red); 
         TakeScreenShot(ticketNum);
      }
      else if(!buy && validBuySignal)
      {
        if(IsInTime())
        {
           ticketNum = OrderSend(Symbol(), OP_BUY, ls, Ask, 3, 0, 0,TradeComment + " - Buy", Magic, 0, Blue);
           if(ticketNum > 0)
           {
              TakeScreenShot(ticketNum);
           }  
        }
      } 
      else if(lastTime != iTime(NULL, 0, 0) && validBuySignal && allProfit && OpenNewTradeWithEachNewBar && IsInTime())
      {
         lastTime = iTime(NULL, 0, 0);
         ticketNum = OrderSend(Symbol(), OP_SELL, ls, Ask, 3, 0, 0,TradeComment + " - Buy", Magic, 0, Red);
         TakeScreenShot(ticketNum);
      }
      else
      {
         CloseAllOrders();
      }
   
    string tradeType = "";
    if(validBuySignal) tradeType = "BUYING!";
    else if(validBuySignal) tradeType = "SELLING!";
    else tradeType = "WAITING";
    string screenComment = StringConcatenate(fibComment, "Trade = ", tradeType, "\r\nProfit = $ " + profit);
    
    if(UseTimeLimits)
    {
      if(IsInTime())
      {
         screenComment = StringConcatenate(screenComment, "\r\nIN Trading Time Frame");
      }
      else
      {
         screenComment = StringConcatenate(screenComment, "\r\nOUT of Trading Time Frame");
      }
    }
    
    if(posSemaphore3Up < posSemaphore3Dn)
    {
      screenComment = StringConcatenate(screenComment, "\r\n", "3 = Sell");
    }
    else
    {
      screenComment = StringConcatenate(screenComment, "\r\n", "3 = BUY");
    }
    
    screenComment = StringConcatenate(screenComment, "\r\n",TimeToStr( TimeLocal()));

    Comment(screenComment);
   
//----
   return(0);
  }
  
void CloseAllOrders()
{
   for(int i=0; i<OrdersTotal(); i++)
   {
      if(OrderSelect(i, SELECT_BY_POS, MODE_TRADES) == true && OrderMagicNumber() == Magic)
      {
         if(OrderSymbol()== Symbol() && OrderType() == OP_SELL)
         {
            OrderClose(OrderTicket(), OrderLots(), Bid, 2);
            TakeScreenShot(OrderTicket());
         }
         if(OrderSymbol() == Symbol() && OrderType() == OP_BUY)
         {
            OrderClose(OrderTicket(), OrderLots(), Ask, 2);
            TakeScreenShot(OrderTicket());
         }
      }
   }
}

bool IsInTime()
{
   if(!UseTimeLimits) return(true);
   if(DayOfWeek() >= StartDay && DayOfWeek() <=EndDay)
   {
      if(StartHour < EndHour)
      {
         if(TimeHour(TimeLocal()) >= StartHour && TimeHour(TimeLocal()) <= EndHour)
         {
            if(TimeHour(TimeLocal()) == StartHour && TimeMinute(TimeLocal()) < StartMin)
            {
               return(false);  
            }
            else if(TimeHour(TimeLocal()) == EndHour && TimeMinute(TimeLocal()) > EndMin)
            {
               return(false);
            }
            return(true);
         }
      }
      else if(StartHour > EndHour)
      {
         if(TimeHour(TimeLocal()) >= StartHour || TimeHour(TimeLocal()) <= EndHour)
         {
            if(TimeHour(TimeLocal()) == StartHour && TimeMinute(TimeLocal()) < StartMin)
            {
               return(false);
            }
            else if(TimeHour(TimeLocal()) == EndHour && TimeMinute(TimeLocal()) > EndMin)
            {
               return(false);
            }
            return(true);
         }
                  
      }
      return(false);
   }  
   return(false);
}

bool OrderTypeExists(int orderType, datetime after)
{
   for(int i=0; i<OrdersTotal(); i++)
   {
      if(OrderSelect(i, SELECT_BY_POS, MODE_TRADES) == true)
      {
         if(OrderMagicNumber() == Magic && OrderSymbol() == Symbol())
         {
            if(OrderType() == orderType)
            { 
               return(true);
            }
         }
      }
      if(OrderSelect(i, SELECT_BY_POS, MODE_HISTORY) == true)
      {
         if(OrderSymbol() == Symbol() && OrderMagicNumber() == Magic)
         {
            if(OrderOpenTime() >= after)
            {
               if(OrderType() == orderType)
               {
                  return(true);
               }

            }
         }
      }
   }
   return(false);
}
  
//+------------------------------------------------------------------+